<?php
/*
 * Plugin Name:       Nitrox
 * Description:       Implements the required tagging blocks for using Nitrox service.
 * Version:           1.0.0
 * Author:            Nitrocommerce
 * Author URI:        https://nitrocommerce.ai/
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       nitrox
 * Requires Plugins:  woocommerce
 */

if (!defined('ABSPATH')) {
    exit;
}

class Nitrox {

    private $s_url = 'https://x.nitrocommerce.ai';
    private $cc_url = 'https://t.makehook.ws';

    public function __construct() {
        add_action('admin_menu', array($this, 'add_admin_menu'));
        add_action('admin_init', array($this, 'settings_init'));

        add_action('wp_head', array($this, 'enqueue_pixel_script'));

        add_action('woocommerce_before_main_content', array($this, 'track_category_view'));
        add_action('woocommerce_after_single_product', array($this, 'track_product_view'));
        add_action('woocommerce_add_to_cart', array($this, 'track_add_to_cart'), 10, 6);
        add_action('woocommerce_remove_cart_item', array($this, 'track_remove_from_cart'), 10, 2);  
        add_action('woocommerce_after_cart_item_quantity_update', array($this, 'track_update_cart_quantity'), 10, 3);
        add_action('template_redirect', array($this, 'track_checkout'));
        add_action('woocommerce_thankyou', array($this, 'track_buy'));

        add_action('user_register', array($this, 'track_user_registration'), 10, 1);
        add_action('wp_login', array($this, 'track_user_login'), 10, 2);
        add_action('woocommerce_order_status_changed', array($this, 'track_user_registration_on_checkout'), 10, 2);
        add_action('wp_footer', array($this, 'identify_user_script'), 9999);
        
        add_action('wp_ajax_add_to_cart_action', array($this, 'handle_collection_add_to_cart_ajax'));
        add_action('wp_ajax_nopriv_add_to_cart_action', array($this, 'handle_collection_add_to_cart_ajax'));

        add_action('wp_enqueue_scripts', array($this, 'enqueue_collection_add_to_cart_script'));

        add_filter( 'script_loader_tag', array($this, 'disable_caching_on_script'), 10, 3 );

    }

    public function enqueue_collection_add_to_cart_script() {
        wp_enqueue_script('nitrox-custom-script', plugin_dir_url(__FILE__) . 'js/nitrox-custom.js', array('jquery'), null, true);
    
        wp_localize_script('nitrox-custom-script', 'customAddToCart', array(
            'ajax_url' => admin_url('admin-ajax.php')
        ));
    }

    public function add_admin_menu() {
        add_options_page('Nitrox', 'Nitrox', 'manage_options', 'nitrox', array($this, 'options_page'));
    }

    public function settings_init() {
        register_setting('nitrox', 'nitrox_settings', array(
            'sanitize_callback' => array($this, 'sanitize_settings')
        ));

        add_settings_section('nitrox_section', __('Settings', 'nitrox'), null, 'nitrox');

        add_settings_field('nitrox_org_token', __('Org Token', 'nitrox'), array($this, 'org_token_render'), 'nitrox', 'nitrox_section');
    }

    public function sanitize_settings($input) {
        $sanitized = array();
        if (isset($input['nitrox_org_token'])) {
            $sanitized['nitrox_org_token'] = sanitize_text_field($input['nitrox_org_token']);
        }
        return $sanitized;
    }

    public function org_token_render() {
        $options = get_option('nitrox_settings');
        $org_token = isset($options['nitrox_org_token']) ? esc_attr($options['nitrox_org_token']) : '';
        ?>
        <input type='text' name='nitrox_settings[nitrox_org_token]' value='<?php echo $org_token; ?>'>
        <?php
    }

    public function options_page() {
        if (!current_user_can('manage_options')) {
            return;
        }
        ?>
        <form action='options.php' method='post'>
            <h2><?php esc_html_e('Nitrox', 'nitrox'); ?></h2>
            <?php
            settings_fields('nitrox');
            do_settings_sections('nitrox');
            submit_button();
            ?>
        </form>
        <?php
    }

    public function enqueue_pixel_script() {
        $options = get_option('nitrox_settings');
        $org_token = isset($options['nitrox_org_token']) ? esc_js($options['nitrox_org_token']) : '';

        if ($org_token) {
            echo '<script data-cfasync="false">' . PHP_EOL;

            $custom_script = "
            (function(n,i,t,r,o) {
                var a,m;n['NitroObject']=o;n[o]=n[o]||function(){
                    (n[o].q=n[o].q||[]).push(arguments)},n[o].l=1*new Date();n[o].h=r;a=i.createElement(t),
                    m=i.getElementsByTagName(t)[0];a.async=1;a.src=r;m.parentNode.insertBefore(a,m)
                    })(window,document,'script','$this->s_url/base.js','nitro');
                    nitro('configure', '$org_token', 'WORDPRESS');
            ";

            echo $custom_script . PHP_EOL;

            echo '</script>' . PHP_EOL;
        }
    }

    public function disable_caching_on_script($tag, $handle, $src) {
        if ('nitrox-custom-script' == $handle) {
            return $tag = '<script data-cfasync="false" src="' . esc_url( $src ) . '"></script>';
        }

        return $tag;
    }

    public function track_add_to_cart($cart_id, $product_id, $request_quantity, $variation_id, $variation, $cart_item_data) {
        $cart = WC()->cart;
        $line_items = array();
        
        foreach ($cart->get_cart() as $cart_item) {
            $product = wc_get_product($cart_item['product_id']);
            $line_items[] = array(
                'quantity' => $cart_item['quantity'],
                'title' => $product->get_name(),
                'line_price' => wc_get_price_to_display($product, array('price' => $product->get_price())),
                'id' => $product->get_id(),
                'product_id' => $product->get_id(),
                'image_url' => wp_get_attachment_url($product->get_image_id())
            );
        }

        $cart->calculate_totals();

        $cart_data = array(
            'line_items' => $line_items,
            'cart_value' => $cart->get_total('edit')
        );

        $cart_data_json = wp_json_encode($cart_data);

        add_action('wp_footer', function() use ($cart_data_json) {
            echo "<script data-cfasync='false'>
                document.addEventListener('DOMContentLoaded', function() {
                    if (window.nitro) {
                        setTimeout(function(){
                            nitro.track('updatecart', $cart_data_json);
                        }, 1500);
                    }
                });
            </script>";
        });
    }

    public function handle_collection_add_to_cart_ajax() {
        global $woocommerce;

        if ( !isset($_POST['product_id']) || !is_numeric($_POST['product_id']) ) {
            wp_send_json_error('Invalid product ID');
            return;
        }
        // $product_id = intval($_POST['product_id']);
        $cart = WC()->cart;
        
        $cart->calculate_totals();

        $line_items = array();

        foreach ( $cart->get_cart() as $cart_item_key => $cart_item ) {
            $product = wc_get_product($cart_item['product_id']);

            $line_items[] = array(
                'quantity' => $cart_item['quantity'],
                'title' => $product->get_name(),
                'line_price' => wc_get_price_to_display($product, array('qty' => $cart_item['quantity'])),
                'id' => $product->get_id(),
                'product_id' => $product->get_id(),
                'image_url' => wp_get_attachment_url($product->get_image_id())
            );
        }
    
        $cart_data = array(
            'line_items' => $line_items,
            'cart_value' => $cart->get_total('edit')
        );
    
        wp_send_json_success($cart_data);
    }

    public function track_remove_from_cart( $cart_item_key, $cart ) {
        global $woocommerce;
        $cart = WC()->cart;
        // $item = $cart->removed_cart_contents[$cart_item_key];
        // $product = wc_get_product($item['product_id']);

        $line_items = array();

        foreach ($cart->get_cart() as $key => $cart_item) {
            if ($key !== $cart_item_key) {
                $product = wc_get_product($cart_item['product_id']);
                $line_items[] = array(
                    'quantity' => $cart_item['quantity'],
                    'title' => $product->get_name(),
                    'line_price' => wc_get_price_to_display($product, array('price' => $product->get_price())),
                    'id' => $product->get_id(),
                    'product_id' => $product->get_id(),
                    'image_url' => wp_get_attachment_url($product->get_image_id())
                );
            }
        }

        $cart->calculate_totals();
        $cart_total_amount = $cart->get_total('edit');

        $remove_cart_data = array(
            'line_items' => $line_items,
            'cart_value' => $cart_total_amount
        );

        $remove_cart_data_json = wp_json_encode($remove_cart_data);
        
        if (isset( $_COOKIE['parleg'] )) {
            $cookieValue = $_COOKIE['parleg'];
            $jsonString = stripslashes($cookieValue);
            $parleg_data = json_decode($jsonString, true);
            $nitro_id = $parleg_data['id'];
            $biscuit_id = $parleg_data['biscuit'];
        }
        
        // if ( $cart_total_amount > 0 ) {
            
            // $body = base64_encode($remove_cart_data_json);
            $body = $remove_cart_data_json;
            $options = get_option('nitrox_settings');
            $org_token = isset($options['nitrox_org_token']) ? esc_attr($options['nitrox_org_token']) : '';
            $roaming_id = $biscuit_id;

            if ( isset( $biscuit_id ) && ! empty( $biscuit_id ) && ! empty( $org_token ) ) :
    
                $endpoint = "$this->cc_url/jsv1/$org_token/$roaming_id/$nitro_id/null/event/updatecart";

                $options = array(
                    'headers'     => array(
                        'Content-Type' => 'application/json',
                        'User-Agent' => 'Mozilla/5.0 (compatible; NitroCommerce/2.1; +https://nitrocommerce.ai)'
                    ),
                    'body' => $body
                );
    
                $response = wp_remote_post( esc_url_raw( $endpoint ), $options );
                
                if ( is_wp_error( $response ) ) :
                    error_log( print_r( $response->get_error_message(), true ) );
                endif;
            endif;
        // }
    }

    public function track_update_cart_quantity($cart_item_key, $quantity, $old_quantity) {
        global $woocommerce;
        $cart = WC()->cart;
        $line_items = array();

        foreach ($cart->get_cart() as $cart_item_key => $cart_item) {
            $product = wc_get_product($cart_item['product_id']);
            $line_items[] = array(
                'quantity' => $cart_item['quantity'],
                'title' => $product->get_name(),
                'line_price' => wc_get_price_to_display($product, array('price' => $product->get_price())),
                'id' => $product->get_id(),
                'product_id' => $product->get_id(),
                'image_url' => wp_get_attachment_url($product->get_image_id())
            );
        }
    
        $cart->calculate_totals();
        $cart_value = $cart->get_total('edit');
    
        $cart_data = array(
            'line_items' => $line_items,
            'cart_value' => $cart_value
        );
    
        $cart_data_json = wp_json_encode($cart_data);
        if (isset( $_COOKIE['parleg'] )) {
            $cookieValue = $_COOKIE['parleg'];
            $jsonString = stripslashes($cookieValue);
            $parleg_data = json_decode($jsonString, true);
            $nitro_id = $parleg_data['id'];
            $biscuit_id = $parleg_data['biscuit'];
        }
        
        // $body = base64_encode($cart_data_json);
        $body = $cart_data_json;
        $options = get_option('nitrox_settings');
        $org_token = isset($options['nitrox_org_token']) ? esc_attr($options['nitrox_org_token']) : '';
        $roaming_id = $biscuit_id;
    
        if ( isset( $biscuit_id ) && ! empty( $biscuit_id ) && ! empty( $org_token ) ) :
    
            $endpoint = "$this->cc_url/jsv1/$org_token/$roaming_id/$nitro_id/null/event/updatecart";
    
            $options = array(
                'headers'     => array(
                    'Content-Type' => 'application/json',
                    'User-Agent' => 'Mozilla/5.0 (compatible; NitroCommerce/2.1; +https://nitrocommerce.ai)'
                ),
                'body' => $body
            );
    
            $response = wp_remote_post( esc_url_raw( $endpoint ), $options );
            if ( is_wp_error( $response ) ) :
                error_log( print_r( $response->get_error_message(), true ) );
            endif;
        endif;
    }

    public function track_checkout() {
        if (is_checkout() && ! is_wc_endpoint_url()) {
            $cart = WC()->cart;
            $checkout_data = array(
                'checkout_url' => esc_url_raw(wc_get_checkout_url()),
                'total_price' => $cart->get_total('edit')
            );

            $checkout_data_json = wp_json_encode($checkout_data);

            add_action('wp_footer', function() use ($checkout_data_json) {
                echo "<script data-cfasync='false'>
                    document.addEventListener('DOMContentLoaded', function() {
                        if (window.nitro) {
                            setTimeout(function(){
                                nitro.checkout($checkout_data_json);
                            }, 1500);
                        }
                    });
                </script>";
            });
        }
    }

    public function track_buy($order_id) {
        if (is_wc_endpoint_url( 'order-received' )) {
            $order = wc_get_order($order_id);
            $buy_data = array(
                'order_status_url' => esc_url_raw($order->get_checkout_order_received_url()),
                'order_id' => $order->get_id(),
                'total_price' => $order->get_total(),
                'currency' => $order->get_currency()
            );

            $buy_data_json = wp_json_encode($buy_data);

            add_action('wp_footer', function() use ($buy_data_json) {
                echo "<script data-cfasync='false'>
                    document.addEventListener('DOMContentLoaded', function() {
                        if (window.nitro) {
                            setTimeout(function(){
                                nitro.buy($buy_data_json);
                            }, 1500);
                        }
                    });
                </script>";
            });
        }
    }

    public function track_product_view() {
        if (is_product()) {
            global $post;
            $product_id = sanitize_text_field($post->ID);
            $product = wc_get_product($product_id);

            $product_data = array(
                'product_id' => $product->get_id(),
                'price' => $product->get_price(),
                'product_url' => get_permalink($product->get_id())
            );

            $product_data_json = wp_json_encode($product_data);

            add_action('wp_footer', function() use ($product_data_json) {
                echo "<script data-cfasync='false'>
                    document.addEventListener('DOMContentLoaded', function() {
                        if (window.nitro) {
                            setTimeout(function(){
                                nitro.productView($product_data_json);
                            }, 1500);
                        }
                    });
                </script>";
            });
        }
    }

    public function track_category_view() {
        if (is_product_category()) {
            $category = get_queried_object();
            $category_data = array(
                'category' => sanitize_text_field($category->name)
            );

            $category_data_json = wp_json_encode($category_data);

            add_action('wp_footer', function() use ($category_data_json) {
                echo "<script data-cfasync='false'>
                    document.addEventListener('DOMContentLoaded', function() {
                        if (window.nitro) {
                            setTimeout(function(){
                                nitro.categoryView($category_data_json);
                            }, 1500);
                        }
                    });
                </script>";
            });
        }
    }

    public function track_user_registration($user_id) {
        $current_user = get_userdata($user_id);
       
        $phone_number = get_user_meta($user_id, 'billing_phone', true) ? get_user_meta($user_id, 'billing_phone', true) : null;
        $email = $current_user->user_email ? $current_user->user_email : null;
        $full_name = $current_user->first_name ? $current_user->first_name . " " . $current_user->last_name : null;

        $transient_data = [
            'email' => $email,
            'phone' => $phone_number,
            'name' => $full_name
        ];
        
        $this->set_nitro_transients($user_id, $transient_data);
    }

    public function track_user_login($user_login, WP_User $user) {
        $user_id = $user->ID;
        $current_user = get_userdata($user_id);
        
        $phone_number = get_user_meta($user_id, 'billing_phone', true) ? get_user_meta($user_id, 'billing_phone', true) : null;
        $email = $current_user->user_email ? $current_user->user_email : null;
        $full_name = $current_user->first_name ? $current_user->first_name . " " . $current_user->last_name : null;
        
        $transient_data = [
            'email' => $email,
            'phone' => $phone_number,
            'name' => $full_name
        ];

        $this->set_nitro_transients($user_id, $transient_data);

    }
    
    public function track_user_registration_on_checkout($order_id, $checkout = null) {
        global $woocommerce;
        $order = wc_get_order($order_id);
    
        if ($order->get_status() === 'processing' || $order->get_status() === 'on-hold') {
            $email = $order->get_billing_email() ? $order->get_billing_email() : null;
            $phone_number = $order->get_billing_phone() ? $order->get_billing_phone() : null;
            $full_name = $order->get_billing_first_name() ? $order->get_billing_first_name() . ' ' . $order->get_billing_last_name() : null;
            
            $transient_data = [
                'email' => $email,
                'phone' => $phone_number,
                'name' => $full_name
            ];

            $this->set_nitro_transients($order_id, $transient_data);
        }
    }

    public function identify_user_script() {
        $identifier = is_user_logged_in() ? get_current_user_id() : (isset($_GET['key']) ? intval(wc_get_order_id_by_order_key($_GET['key'])) : null);
        
        if ($identifier && true == get_transient("nitro_userdata_$identifier")) {
            $email = get_transient("nitro_userdata_email_$identifier");
            $phone = get_transient("nitro_userdata_phone_$identifier");
            $currentUserName = get_transient("nitro_userdata_name_$identifier");

            $identifyScript = "<script data-cfasync='false'>
                document.addEventListener('DOMContentLoaded', function() {
                    if (window.nitro) {
                        setTimeout(function(){
                            var email = " . ($email ? "'$email'" : 'null') . ";
                            var phone = " . ($phone ? "'$phone'" : 'null') . ";
                            var name = " . ($currentUserName ? "'$currentUserName'" : 'null') . ";
                            nitro.identify(email, phone, name);
                        }, 1500);
                    }
                });
            </script>";
            
            echo $identifyScript;
    
            $this->delete_nitro_transients($identifier);
        }
    }

    private function set_nitro_transients($identifier, $data) {

        $email = $data['email'];
        $phone_number = $data['phone'];
        $full_name = $data['name'];

        set_transient("nitro_userdata_email_$identifier", $email);
        set_transient("nitro_userdata_phone_$identifier", $phone_number);
        set_transient("nitro_userdata_name_$identifier", $full_name);
        set_transient("nitro_userdata_$identifier", true);
    }

    private function delete_nitro_transients($identifier) {
        delete_transient("nitro_userdata_email_$identifier");
        delete_transient("nitro_userdata_phone_$identifier");
        delete_transient("nitro_userdata_name_$identifier");
        delete_transient("nitro_userdata_$identifier");
    }
    
}

new Nitrox();
?>